@@ -91,8 +91,8 @@ describe Thor do
 
   describe "#stop_on_unknown_option!" do
     my_script = Class.new(Thor) do
-      class_option "verbose",   :type => :boolean
-      class_option "mode",      :type => :string
+      class_option "verbose",   type: :boolean
+      class_option "mode",      type: :string
 
       stop_on_unknown_option! :exec
 
@@ -140,9 +140,9 @@ describe Thor do
         stop_on_unknown_option! :foo, :bar
       end
       it "affects all specified commands" do
-        expect(klass.stop_on_unknown_option?(double(:name => "foo"))).to be true
-        expect(klass.stop_on_unknown_option?(double(:name => "bar"))).to be true
-        expect(klass.stop_on_unknown_option?(double(:name => "baz"))).to be false
+        expect(klass.stop_on_unknown_option?(double(name: "foo"))).to be true
+        expect(klass.stop_on_unknown_option?(double(name: "bar"))).to be true
+        expect(klass.stop_on_unknown_option?(double(name: "baz"))).to be false
       end
     end
 
@@ -152,9 +152,9 @@ describe Thor do
         stop_on_unknown_option! :bar
       end
       it "affects all specified commands" do
-        expect(klass.stop_on_unknown_option?(double(:name => "foo"))).to be true
-        expect(klass.stop_on_unknown_option?(double(:name => "bar"))).to be true
-        expect(klass.stop_on_unknown_option?(double(:name => "baz"))).to be false
+        expect(klass.stop_on_unknown_option?(double(name: "foo"))).to be true
+        expect(klass.stop_on_unknown_option?(double(name: "bar"))).to be true
+        expect(klass.stop_on_unknown_option?(double(name: "baz"))).to be false
       end
     end
 
@@ -164,8 +164,8 @@ describe Thor do
 
     context "along with check_unknown_options!" do
       my_script2 = Class.new(Thor) do
-        class_option "verbose",   :type => :boolean
-        class_option "mode",      :type => :string
+        class_option "verbose",   type: :boolean
+        class_option "mode",      type: :string
         check_unknown_options!
         stop_on_unknown_option! :exec
 
@@ -219,8 +219,8 @@ describe Thor do
 
   describe "#check_unknown_options!" do
     my_script = Class.new(Thor) do
-      class_option "verbose",   :type => :boolean
-      class_option "mode",      :type => :string
+      class_option "verbose",   type: :boolean
+      class_option "mode",      type: :string
       check_unknown_options!
 
       desc "checked", "a command with checked"
@@ -280,7 +280,7 @@ describe Thor do
 
   describe "#disable_required_check!" do
     my_script = Class.new(Thor) do
-      class_option "foo", :required => true
+      class_option "foo", required: true
 
       disable_required_check! :boring
 
@@ -309,7 +309,7 @@ describe Thor do
     end
 
     it "does affects help by default" do
-      expect(my_script.disable_required_check?(double(:name => "help"))).to be true
+      expect(my_script.disable_required_check?(double(name: "help"))).to be true
     end
 
     context "when provided with multiple command names" do
@@ -318,10 +318,10 @@ describe Thor do
       end
 
       it "affects all specified commands" do
-        expect(klass.disable_required_check?(double(:name => "help"))).to be true
-        expect(klass.disable_required_check?(double(:name => "foo"))).to be true
-        expect(klass.disable_required_check?(double(:name => "bar"))).to be true
-        expect(klass.disable_required_check?(double(:name => "baz"))).to be false
+        expect(klass.disable_required_check?(double(name: "help"))).to be true
+        expect(klass.disable_required_check?(double(name: "foo"))).to be true
+        expect(klass.disable_required_check?(double(name: "bar"))).to be true
+        expect(klass.disable_required_check?(double(name: "baz"))).to be false
       end
     end
 
@@ -332,10 +332,10 @@ describe Thor do
       end
 
       it "affects all specified commands" do
-        expect(klass.disable_required_check?(double(:name => "help"))).to be true
-        expect(klass.disable_required_check?(double(:name => "foo"))).to be true
-        expect(klass.disable_required_check?(double(:name => "bar"))).to be true
-        expect(klass.disable_required_check?(double(:name => "baz"))).to be false
+        expect(klass.disable_required_check?(double(name: "help"))).to be true
+        expect(klass.disable_required_check?(double(name: "foo"))).to be true
+        expect(klass.disable_required_check?(double(name: "bar"))).to be true
+        expect(klass.disable_required_check?(double(name: "baz"))).to be false
       end
     end
   end
@@ -395,7 +395,7 @@ describe Thor do
         expect(capture(:stdout) { MyScript.start(%w(help)) }).not_to match(/this is hidden/m)
       end
 
-      it "but the command is still invokable, does not show the command in help" do
+      it "but the command is still invocable, does not show the command in help" do
         expect(MyScript.start(%w(hidden yesyes))).to eq(%w(yesyes))
       end
     end
@@ -422,6 +422,26 @@ describe Thor do
     end
   end
 
+  describe "#method_exclusive" do
+    it "returns the exclusive option names for the class" do
+      cmd =  MyOptionScript.commands["exclusive"]
+      exclusives = cmd.options_relation[:exclusive_option_names]
+      expect(exclusives.size).to be(2)
+      expect(exclusives.first).to eq(%w[one two three])
+      expect(exclusives.last).to eq(%w[after1 after2])
+    end
+  end
+
+  describe "#method_at_least_one" do
+    it "returns the at least one of option names for the class" do
+      cmd =  MyOptionScript.commands["at_least_one"]
+      at_least_ones = cmd.options_relation[:at_least_one_option_names]
+      expect(at_least_ones.size).to be(2)
+      expect(at_least_ones.first).to eq(%w[one two three])
+      expect(at_least_ones.last).to eq(%w[after1 after2])
+    end
+  end
+
   describe "#start" do
     it "calls a no-param method when no params are passed" do
       expect(MyScript.start(%w(zoo))).to eq(true)
@@ -496,13 +516,13 @@ Usage: "thor scripts:arities:multiple_usages ARG --foo"
       it "raises an exception and displays a message that explains the ambiguity" do
         shell = Thor::Base.shell.new
         expect(shell).to receive(:error).with("Ambiguous command call matches [call_myself_with_wrong_arity, call_unexistent_method]")
-        MyScript.start(%w(call), :shell => shell)
+        MyScript.start(%w(call), shell: shell)
       end
 
       it "raises an exception when there is an alias" do
         shell = Thor::Base.shell.new
         expect(shell).to receive(:error).with("Ambiguous command f matches [foo, fu]")
-        MyScript.start(%w(f), :shell => shell)
+        MyScript.start(%w(f), shell: shell)
       end
     end
   end
@@ -526,7 +546,7 @@ Usage: "thor scripts:arities:multiple_usages ARG --foo"
       end
 
       it "uses the maximum terminal size to show commands" do
-        expect(@shell).to receive(:terminal_width).and_return(80)
+        expect(Thor::Shell::Terminal).to receive(:terminal_width).and_return(80)
         content = capture(:stdout) { MyScript.help(shell) }
         expect(content).to match(/aaa\.\.\.$/)
       end
@@ -550,6 +570,26 @@ Usage: "thor scripts:arities:multiple_usages ARG --foo"
         content = capture(:stdout) { Scripts::MyScript.help(shell) }
         expect(content).to match(/zoo ACCESSOR \-\-param\=PARAM/)
       end
+
+      it "prints class exclusive options" do
+        content = capture(:stdout) { MyClassOptionScript.help(shell) }
+        expect(content).to match(/Exclusive Options:\n\s+--one\s+--two\n/)
+      end
+
+      it "does not print class exclusive options" do
+        content = capture(:stdout) { Scripts::MyScript.help(shell) }
+        expect(content).not_to match(/Exclusive Options:/)
+      end
+
+      it "prints class at least one of requred options" do
+        content = capture(:stdout) { MyClassOptionScript.help(shell) }
+        expect(content).to match(/Required At Least One:\n\s+--three\s+--four\n/)
+      end
+
+      it "does not print class at least one of required options" do
+        content = capture(:stdout) { Scripts::MyScript.help(shell) }
+        expect(content).not_to match(/Required At Least One:/)
+      end
     end
 
     describe "for a specific command" do
@@ -602,11 +642,41 @@ Description:
 HELP
       end
 
+      it "prints long description unwrapped if asked for" do
+        expect(capture(:stdout) { MyScript.command_help(shell, "long_description_unwrapped") }).to eq(<<-HELP)
+Usage:
+  thor my_script:long_description
+
+Description:
+No added indentation,   Inline
+whatespace not merged,
+Linebreaks preserved
+  and
+    indentation
+  too
+HELP
+      end
+
       it "doesn't assign the long description to the next command without one" do
         expect(capture(:stdout) do
           MyScript.command_help(shell, "name_with_dashes")
         end).not_to match(/so very long/i)
       end
+
+      it "prints exclusive and at least one options" do
+        message = expect(capture(:stdout) do
+                           MyClassOptionScript.command_help(shell, "mix")
+                         end)
+        message.to match(/Exclusive Options:\n\s+--five\s+--six\s+--seven\n\s+--one\s+--two/)
+        message.to match(/Required At Least One:\n\s+--five\s+--six\s+--seven\n\s+--three\s+--four/)
+      end
+      it "does not print exclusive and at least one options" do
+        message = expect(capture(:stdout) do
+                           MyOptionScript.command_help(shell, "no_relations")
+                         end)
+        message.not_to match(/Exclusive Options:/)
+        message.not_to match(/Rquired At Least One:/)
+      end
     end
 
     describe "instance method" do
@@ -622,7 +692,7 @@ HELP
     context "with required class_options" do
       let(:klass) do
         Class.new(Thor) do
-          class_option :foo, :required => true
+          class_option :foo, required: true
 
           desc "bar", "do something"
           def bar; end
@@ -667,7 +737,7 @@ HELP
   describe "edge-cases" do
     it "can handle boolean options followed by arguments" do
       klass = Class.new(Thor) do
-        method_option :loud, :type => :boolean
+        method_option :loud, type: :boolean
         desc "hi NAME", "say hi to name"
         def hi(name)
           name = name.upcase if options[:loud]
@@ -680,6 +750,22 @@ HELP
       expect(klass.start(%w(hi --loud jose))).to eq("Hi JOSE")
     end
 
+    it "method_option raises an ArgumentError if name is not a Symbol or String" do
+      expect do
+        Class.new(Thor) do
+          method_option loud: true, type: :boolean
+        end
+      end.to raise_error(ArgumentError, "Expected a Symbol or String, got {:loud=>true, :type=>:boolean}")
+    end
+
+    it "class_option raises an ArgumentError if name is not a Symbol or String" do
+      expect do
+        Class.new(Thor) do
+          class_option loud: true, type: :boolean
+        end
+      end.to raise_error(ArgumentError, "Expected a Symbol or String, got {:loud=>true, :type=>:boolean}")
+    end
+
     it "passes through unknown options" do
       klass = Class.new(Thor) do
         desc "unknown", "passing unknown options"
@@ -725,7 +811,7 @@ HELP
     it "issues a deprecation warning on incompatible types by default" do
       expect do
         Class.new(Thor) do
-          option "bar", :type => :numeric, :default => "foo"
+          option "bar", type: :numeric, default: "foo"
         end
       end.to output(/^Deprecation warning/).to_stderr
     end
@@ -735,7 +821,7 @@ HELP
         Class.new(Thor) do
           allow_incompatible_default_type!
 
-          option "bar", :type => :numeric, :default => "foo"
+          option "bar", type: :numeric, default: "foo"
         end
       end.not_to output.to_stderr
     end
@@ -743,7 +829,7 @@ HELP
     it "allows incompatible types if `check_default_type: false` is given" do
       expect do
         Class.new(Thor) do
-          option "bar", :type => :numeric, :default => "foo", :check_default_type => false
+          option "bar", type: :numeric, default: "foo", check_default_type: false
         end
       end.not_to output.to_stderr
     end
@@ -753,7 +839,7 @@ HELP
         Class.new(Thor) do
           check_default_type!
 
-          option "bar", :type => :numeric, :default => "foo"
+          option "bar", type: :numeric, default: "foo"
         end
       end.to raise_error(ArgumentError, "Expected numeric default value for '--bar'; got \"foo\" (string)")
     end
